@@ -3,20 +3,21 @@ Main entrypoint for builder.
 """
 import sys
 from collections.abc import Iterable
+from typing import Sequence
 
 from boto3.session import Session
 
-from mypy_boto3_builder.aiobotocore_generator import AioBotocoreGenerator
-from mypy_boto3_builder.boto3_generator import Boto3Generator
-from mypy_boto3_builder.cli_parser import parse_args
-from mypy_boto3_builder.constants import DUMMY_REGION, Product
+from mypy_boto3_builder.cli_parser import Namespace, parse_args
+from mypy_boto3_builder.constants import DUMMY_REGION, Product, ProductLibrary
+from mypy_boto3_builder.generators.aiobotocore_generator import AioBotocoreGenerator
+from mypy_boto3_builder.generators.base_generator import BaseGenerator
+from mypy_boto3_builder.generators.boto3_generator import Boto3Generator
 from mypy_boto3_builder.jinja_manager import JinjaManager
 from mypy_boto3_builder.logger import get_logger
-from mypy_boto3_builder.parsers.boto3_utils import get_boto3_resource
 from mypy_boto3_builder.service_name import ServiceName, ServiceNameCatalog
 from mypy_boto3_builder.utils.botocore_changelog import BotocoreChangelog
 from mypy_boto3_builder.utils.strings import get_anchor_link, get_botocore_class_name
-from mypy_boto3_builder.utils.version import get_botocore_version
+from mypy_boto3_builder.utils.version import get_botocore_version, get_builder_version
 
 
 def get_selected_service_names(
@@ -45,7 +46,8 @@ def get_selected_service_names(
         return list(available)
     if ServiceName.UPDATED in selected_service_names:
         selected_service_names.remove(ServiceName.UPDATED)
-        for updated_service_name in BotocoreChangelog().get_updated_service_names(botocore_version):
+        updated_service_names = BotocoreChangelog().get_updated_service_names(botocore_version)
+        for updated_service_name in updated_service_names:
             if updated_service_name in available_map:
                 selected_service_names.append(updated_service_name)
 
@@ -80,6 +82,40 @@ def get_available_service_names(session: Session) -> list[ServiceName]:
     return result
 
 
+def generate_product(
+    product: Product,
+    args: Namespace,
+    session: Session,
+    service_names: Sequence[ServiceName],
+    master_service_names: Sequence[ServiceName],
+) -> None:
+    """
+    Generate a selected product.
+
+    Arguments:
+        product -- Product to generate
+        args -- CLI namespace
+        session -- Boto3 session
+        service_names -- Selected service names
+        master_service_names -- Service names included in master
+    """
+    generator_cls = {
+        ProductLibrary.boto3: Boto3Generator,
+        ProductLibrary.aiobotocore: AioBotocoreGenerator,
+    }[product.get_library()]
+    generator: BaseGenerator = generator_cls(
+        service_names=service_names,
+        master_service_names=master_service_names,
+        session=session,
+        output_path=args.output_path,
+        generate_setup=not args.installed,
+        skip_published=args.skip_published,
+        disable_smart_version=args.disable_smart_version,
+        version=args.build_version,
+    )
+    generator.generate_product(product.get_type())
+
+
 def main() -> None:
     """
     Main entrypoint for builder.
@@ -103,53 +139,20 @@ def main() -> None:
         return
 
     service_names = get_selected_service_names(args.service_names, available_service_names)
-
-    for service_name in available_service_names:
-        service_name.has_service_resource = bool(get_boto3_resource(session, service_name))
-    for service_name in service_names:
-        service_name.has_service_resource = bool(get_boto3_resource(session, service_name))
+    master_service_names = service_names if args.partial_overload else available_service_names
 
     JinjaManager.update_globals(
-        builder_version=args.builder_version,
+        builder_version=get_builder_version(),
         get_anchor_link=get_anchor_link,
         render_docstrings=True,
         hasattr=hasattr,
         len=len,
+        sorted=sorted,
     )
 
-    boto3_generator = Boto3Generator(
-        service_names=service_names,
-        available_service_names=available_service_names,
-        master_service_names=service_names if args.partial_overload else available_service_names,
-        session=session,
-        output_path=args.output_path,
-        generate_setup=not args.installed,
-        skip_published=args.skip_published,
-        disable_smart_version=args.disable_smart_version,
-        version=args.build_version,
-    )
-    aiobotocore_generator = AioBotocoreGenerator(
-        service_names=service_names,
-        available_service_names=available_service_names,
-        master_service_names=service_names if args.partial_overload else available_service_names,
-        session=session,
-        output_path=args.output_path,
-        generate_setup=not args.installed,
-        skip_published=args.skip_published,
-        disable_smart_version=args.disable_smart_version,
-        version=args.build_version,
-    )
-    generators_map = {
-        Product.boto3: boto3_generator.generate_stubs,
-        Product.boto3_services: boto3_generator.generate_service_stubs,
-        Product.boto3_docs: boto3_generator.generate_docs,
-        Product.aiobotocore: aiobotocore_generator.generate_stubs,
-        Product.aiobotocore_services: aiobotocore_generator.generate_service_stubs,
-        Product.aiobotocore_docs: aiobotocore_generator.generate_docs,
-    }
-    generators = [generators_map[p] for p in args.products]
-    for generator in generators:
-        generator()
+    for product in args.products:
+        logger.info(f"Generating {product} product")
+        generate_product(product, args, session, service_names, master_service_names)
 
     logger.info("Completed")
 
